# customer-js-sdk

[![Travis][build-badge]][build]
[![npm package][npm-badge]][npm]
[![Coveralls][coveralls-badge]][coveralls]

The customer-js-sdk is a front-end layer of abstraction for
communicating with all kinds of back-end services. It's
usable in any framework in the front-end, now and in the future.

## Pattern
We decided to use a pattern of modules to use the SDK
for being simple to understand and easy to use.

## Install
```bash
npm install --save customer-js-sdk
```
### Import
```javascript
import { v0 as sdk } from 'customer-js-sdk';
```
### Usage
To make any endpoint call just use the module structure that SDK is created.

```javascript
sdk.merchants.partners.get();

sdk.merchants.partners.get('https://custom.api/');

sdk.merchants.addresses.get({
  urlParam: '123456789'
});

sdk.merchants.addresses.get('https://custom.api/', {
  urlParam: '123456789'
});

sdk.merchants.contacts.post({}, {
  body: '123456789'
});

sdk.merchants.contacts.post('https://custom.api/', {}, {
  body: '123456789'
});
```

### Using Middlewares
Middlewares run before and after every request in the order they were declared.

```javascript
sdk.middleware.use({
  beforeRequest({ proceed, quit }) {
    if (conditionMet) {
      proceed();
    } else {
      quit();
    }
  },
  afterRequest({ proceed }) {
    logSomething();
    proceed();
  }
});

sdk.middleware.use({
  async beforeRequest({ proceed }) {
    await doSomethingAsync();
    proceed();
  }
});
```

Note that every middle must call either `proceed` or `quit` in order to work.

### More information in  [Wiki](https://github.com/stone-payments/customer-js-sdk/wiki).

[build-badge]: https://img.shields.io/travis/user/repo/master.png?style=flat-square
[build]: https://travis-ci.org/user/repo

[npm-badge]: https://img.shields.io/npm/v/npm-package.png?style=flat-square
[npm]: https://www.npmjs.org/package/npm-package

[coveralls-badge]: https://img.shields.io/coveralls/user/repo/master.png?style=flat-square
[coveralls]: https://coveralls.io/github/user/repo
