"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getMerchantFees = exports.updateBankAccounts = exports.getMerchantBankInfo = exports.patchMerchantInfo = exports.getMerchantInfo = exports.getMerchantAddresses = exports.getMerchantPartners = exports.putMerchantContacts = exports.getMerchantContacts = void 0;

var merchantService = _interopRequireWildcard(require("../../core/services/merchantService.js"));

var merchantModel = _interopRequireWildcard(require("../../models/merchantModels.js"));

var transactionsService = _interopRequireWildcard(require("../../core/services/transactionsService.js"));

function _interopRequireWildcard(obj) {
  if (obj && obj.__esModule) {
    return obj;
  } else {
    var newObj = {};

    if (obj != null) {
      for (var key in obj) {
        if (Object.prototype.hasOwnProperty.call(obj, key)) {
          var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {};

          if (desc.get || desc.set) {
            Object.defineProperty(newObj, key, desc);
          } else {
            newObj[key] = obj[key];
          }
        }
      }
    }

    newObj.default = obj;
    return newObj;
  }
}

var getMerchantContacts = function getMerchantContacts(requestParams, instance) {
  return function (dispatch) {
    merchantService.getContacts(requestParams, instance).then(function (response) {
      dispatch({
        type: 'GET_MERCHANT_CONTACTS',
        data: merchantModel.contactsModel(response.data)
      });
    });
  };
};

exports.getMerchantContacts = getMerchantContacts;

var putMerchantContacts = function putMerchantContacts(requestParams, instance) {
  return function (dispatch) {
    merchantService.putContacts(requestParams, instance).then(function (response) {
      dispatch({
        type: 'PUT_MERCHANT_CONTACTS',
        data: response.data
      });
    });
  };
};

exports.putMerchantContacts = putMerchantContacts;

var getMerchantPartners = function getMerchantPartners(requestParams, instance) {
  return function (dispatch) {
    merchantService.getPartners(requestParams, instance).then(function (response) {
      dispatch({
        type: 'GET_MERCHANT_PARTNERS',
        data: merchantModel.partnersModel(response)
      });
    });
  };
};

exports.getMerchantPartners = getMerchantPartners;

var getMerchantAddresses = function getMerchantAddresses(requestParams, instance) {
  return function (dispatch) {
    merchantService.getAddresses(requestParams, instance).then(function (response) {
      dispatch({
        type: 'GET_MERCHANT_ADDRESSES',
        data: merchantModel.addressesModel(response.data)
      });
    });
  };
};

exports.getMerchantAddresses = getMerchantAddresses;

var getMerchantInfo = function getMerchantInfo(requestParams, instance) {
  return function (dispatch) {
    merchantService.getInfo(requestParams, instance).then(function (response) {
      dispatch({
        type: 'GET_MERCHANT_INFO',
        data: merchantModel.infoModel(response.data, requestParams.stonecode)
      });
    });
  };
};

exports.getMerchantInfo = getMerchantInfo;

var patchMerchantInfo = function patchMerchantInfo(requestParams, instance) {
  return function (dispatch) {
    merchantService.patchInfo(requestParams, instance).then(function (response) {
      dispatch({
        type: 'PATCH_MERCHANT_FANTASY_NAME',
        data: response.data.fantasyName
      });
    });
  };
};

exports.patchMerchantInfo = patchMerchantInfo;

var getMerchantBankInfo = function getMerchantBankInfo(requestParams, instance) {
  return function (dispatch) {
    var promises = [merchantService.getAvailableBanks(requestParams, instance), merchantService.getBankInfo(requestParams, instance)];
    Promise.all(promises).then(function (responses) {
      dispatch({
        type: 'GET_MERCHANT_BANK_INFO',
        data: merchantModel.bankInfoModel(responses)
      });
    });
  };
};

exports.getMerchantBankInfo = getMerchantBankInfo;

var updateBankAccounts = function updateBankAccounts(requestParams, instance) {
  return function (dispatch) {
    merchantService.putBankAccounts(requestParams, instance).then(function (response) {
      dispatch({
        type: 'PUT_MERCHANT_BANK_INFO',
        data: response.data
      });
    });
  };
};

exports.updateBankAccounts = updateBankAccounts;

var getMerchantFees = function getMerchantFees(requestParams, instance) {
  return function (dispatch) {
    var promises = [transactionsService.getTransactionProfiles(requestParams, instance), transactionsService.getBrands(requestParams, instance), merchantService.getFees(requestParams, instance)];
    Promise.all(promises).then(function (responses) {
      dispatch({
        type: 'GET_MERCHANT_FEES',
        data: merchantModel.feesModel(responses)
      });
    });
  };
};

exports.getMerchantFees = getMerchantFees;