"use strict";

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.patchRequest = exports.putRequest = exports.postRequest = exports.getRequest = exports.request = void 0;

var _makeRequest = require("./makeRequest.js");

var _uniqId = require("./uniqId.js");

var _baseUrl = require("../services/baseUrl.js");

var _headers = require("../services/headers.js");

var _validateRequiredParams = require("./validateRequiredParams.js");

var _validateBaseUrl = require("./validateBaseUrl.js");

var _validatePayload = require("./validatePayload.js");

var _middleware = require("../services/middleware/middleware.js");

var _constants = require("../constants/constants.js");

var request = function request(urlMaker) {
  var method = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : _constants.GET;
  var requestInterface = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var requestConfig = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
  var fetchMaker = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : _makeRequest.makeRequest;
  return function () {
    try {
      for (var _len = arguments.length, userArgs = new Array(_len), _key = 0; _key < _len; _key++) {
        userArgs[_key] = arguments[_key];
      }

      var _userArgs$filter = userArgs.filter(function (arg) {
        return _typeof(arg) === 'object';
      }),
          _userArgs$filter2 = _slicedToArray(_userArgs$filter, 2),
          _userArgs$filter2$ = _userArgs$filter2[0],
          urlParams = _userArgs$filter2$ === void 0 ? {} : _userArgs$filter2$,
          _userArgs$filter2$2 = _userArgs$filter2[1],
          payload = _userArgs$filter2$2 === void 0 ? {} : _userArgs$filter2$2;

      var userBaseUrl = userArgs.find(function (arg) {
        return typeof arg === 'string';
      }) || (0, _baseUrl.get)();
      var resource = (0, _validateBaseUrl.validateBaseUrl)(userBaseUrl);
      var url = typeof urlMaker === 'function' ? urlMaker(urlParams) : urlMaker || '';
      var validUrl = (0, _validateRequiredParams.validateRequiredParams)("".concat(resource).concat(url));
      var requestParams = {
        method: method,
        headers: (0, _headers.get)()
      };

      if (Object.keys(payload).length > 0) {
        requestParams = _objectSpread({}, requestParams, {
          body: (0, _validatePayload.validatePayload)(requestInterface, payload, method)
        });
      }

      var requestId = (0, _uniqId.uniqId)();

      var executeRequest = function executeRequest() {
        return fetchMaker(validUrl, requestParams, requestConfig);
      };

      return Promise.resolve().then(_middleware.middleware.executePreRequest(requestId)).then(function (signal) {
        return signal !== _middleware.INTERRUPTED ? executeRequest().then(_middleware.middleware.executePostRequest(requestId)) : undefined;
      });
    } catch (err) {
      return Promise.reject(err);
    }
  };
};

exports.request = request;

var getRequest = function getRequest(urlMaker) {
  for (var _len2 = arguments.length, args = new Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
    args[_key2 - 1] = arguments[_key2];
  }

  return request.apply(void 0, [urlMaker, _constants.GET, undefined].concat(args));
};

exports.getRequest = getRequest;

var postRequest = function postRequest(urlMaker, requestInterface) {
  for (var _len3 = arguments.length, args = new Array(_len3 > 2 ? _len3 - 2 : 0), _key3 = 2; _key3 < _len3; _key3++) {
    args[_key3 - 2] = arguments[_key3];
  }

  return request.apply(void 0, [urlMaker, _constants.POST, requestInterface].concat(args));
};

exports.postRequest = postRequest;

var putRequest = function putRequest(urlMaker, requestInterface) {
  for (var _len4 = arguments.length, args = new Array(_len4 > 2 ? _len4 - 2 : 0), _key4 = 2; _key4 < _len4; _key4++) {
    args[_key4 - 2] = arguments[_key4];
  }

  return request.apply(void 0, [urlMaker, _constants.PUT, requestInterface].concat(args));
};

exports.putRequest = putRequest;

var patchRequest = function patchRequest(urlMaker, requestInterface) {
  for (var _len5 = arguments.length, args = new Array(_len5 > 2 ? _len5 - 2 : 0), _key5 = 2; _key5 < _len5; _key5++) {
    args[_key5 - 2] = arguments[_key5];
  }

  return request.apply(void 0, [urlMaker, _constants.PATCH, requestInterface].concat(args));
};

exports.patchRequest = patchRequest;