"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.patchRequest = exports.putRequest = exports.postRequest = exports.getRequest = exports.request = void 0;

var _makeRequest = require("./makeRequest.js");

var _uniqId = require("./uniqId.js");

var _baseUrl = require("../services/baseUrl.js");

var _headers = require("../services/headers.js");

var _validateRequiredParams = require("./validateRequiredParams.js");

var _validateBaseUrl = require("./validateBaseUrl.js");

var _validatePayload = require("./validatePayload.js");

var _middleware = require("../services/middleware/middleware.js");

var _constants = require("../constants/constants.js");

const request = (urlMaker, method = _constants.GET, requestInterface = {}, requestConfig = {}, fetchMaker = _makeRequest.makeRequest) => (...userArgs) => {
  try {
    const [urlParams = {}, payload = {}] = userArgs.filter(arg => typeof arg === 'object');
    const userBaseUrl = userArgs.find(arg => typeof arg === 'string') || (0, _baseUrl.get)();
    const resource = (0, _validateBaseUrl.validateBaseUrl)(userBaseUrl);
    const url = typeof urlMaker === 'function' ? urlMaker(urlParams) : urlMaker || '';
    const validUrl = (0, _validateRequiredParams.validateRequiredParams)(`${resource}${url}`);
    let requestParams = {
      method,
      headers: (0, _headers.get)()
    };

    if (Object.keys(payload).length > 0) {
      requestParams = { ...requestParams,
        body: (0, _validatePayload.validatePayload)(requestInterface, payload, method)
      };
    }

    const requestId = (0, _uniqId.uniqId)();

    const executeRequest = () => fetchMaker(validUrl, requestParams, requestConfig);

    return Promise.resolve().then(_middleware.middleware.executePreRequest(requestId)).then(signal => signal !== _middleware.INTERRUPTED ? executeRequest().then(_middleware.middleware.executePostRequest(requestId)) : undefined);
  } catch (err) {
    return Promise.reject(err);
  }
};

exports.request = request;

const getRequest = (urlMaker, ...args) => request(urlMaker, _constants.GET, undefined, ...args);

exports.getRequest = getRequest;

const postRequest = (urlMaker, requestInterface, ...args) => request(urlMaker, _constants.POST, requestInterface, ...args);

exports.postRequest = postRequest;

const putRequest = (urlMaker, requestInterface, ...args) => request(urlMaker, _constants.PUT, requestInterface, ...args);

exports.putRequest = putRequest;

const patchRequest = (urlMaker, requestInterface, ...args) => request(urlMaker, _constants.PATCH, requestInterface, ...args);

exports.patchRequest = patchRequest;