"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
Object.defineProperty(exports, "INTERRUPTED", {
  enumerable: true,
  get: function () {
    return _constants.INTERRUPTED;
  }
});
exports.middleware = exports.Middleware = void 0;

var _constants = require("./constants.js");

var _validateMiddleware = require("./validateMiddleware.js");

class Middleware {
  constructor() {
    this._index = 0;
    this._middlewares = [];
    this._requests = {};
    this.executePreRequest = this.executePreRequest.bind(this);
    this.executePostRequest = this.executePostRequest.bind(this);
    this._proceed = this._proceed.bind(this);
    this._quit = this._quit.bind(this);
  }

  getAll() {
    return this._middlewares;
  }

  use(middleware) {
    this._middlewares.push((0, _validateMiddleware.validateMiddleware)(middleware));
  }

  executePreRequest(requestId) {
    return arg => this._exec(_constants.BEFORE_REQUEST, requestId, arg);
  }

  executePostRequest(requestId) {
    return arg => this._exec(_constants.AFTER_REQUEST, requestId, arg);
  }

  _exec(stage, requestId, arg) {
    this._requests[requestId] = this._requests[requestId] || {};
    this._requests[requestId].stage = stage;

    this._executeCurrentMiddleware(requestId);

    return new Promise(resolve => {
      this._requests[requestId].resolver = signal => {
        resolve(signal || arg);
      };
    });
  }

  _executeCurrentMiddleware(requestId) {
    const currentMiddleware = this._middlewares[this._index];
    const {
      stage
    } = this._requests[requestId];

    if (!currentMiddleware || !currentMiddleware[stage]) {
      this._proceed(requestId);

      return undefined;
    }

    const currentCallback = currentMiddleware[stage];
    const signal = stage === _constants.BEFORE_REQUEST ? _constants.INTERRUPTED : undefined;
    currentCallback({
      proceed: () => this._proceed(requestId),
      quit: () => this._quit(requestId, signal)
    });
    return undefined;
  }

  _proceed(requestId) {
    if (this._index + 1 < this._middlewares.length) {
      this._index += 1;

      this._executeCurrentMiddleware(requestId);

      return undefined;
    }

    this._quit(requestId);

    return undefined;
  }

  _quit(requestId, signal) {
    Promise.resolve().then(() => {
      this._requests[requestId].resolver(signal);

      this._requests[requestId] = undefined;
      this._index = 0;
    });
  }

}

exports.Middleware = Middleware;
const middleware = new Middleware();
exports.middleware = middleware;