import { onSdkAvailable } from '../../connect/onSdkAvailable.js';
import { authorization, headers, BEARER } from '../../v0/src/index.js';

var startLoader = function startLoader(loaderInstance) {
  onSdkAvailable(function ($sdk) {
    $sdk.dispatch({
      type: 'START_LOADER',
      loaderInstance: loaderInstance
    });
  });
};

var stopLoader = function stopLoader(loaderInstance) {
  onSdkAvailable(function ($sdk) {
    $sdk.dispatch({
      type: 'STOP_LOADER',
      loaderInstance: loaderInstance
    });
  });
};

var errorHandler = function errorHandler(instance, responseOrError) {
  var status = responseOrError.status,
      statusText = responseOrError.statusText,
      data = responseOrError.data;

  if (status != null && statusText != null) {
    if (status >= 400) {
      onSdkAvailable(function ($sdk) {
        $sdk.dispatch({
          type: 'SET_INSTANCE_NETWORK_ERROR',
          instance: instance
        });
      });
    } else if (data != null && Array.isArray(data.messages) && data.messages.length > 0) {
      onSdkAvailable(function ($sdk) {
        $sdk.dispatch({
          type: 'SET_INSTANCE_ERROR',
          instance: instance,
          errors: data.messages
        });
      });
    } else {
      onSdkAvailable(function ($sdk) {
        $sdk.dispatch({
          type: 'CLEAN_INSTANCE_ERROR',
          instance: instance
        });
      });
    }
  }

  return undefined;
};

var requestService = function requestService(requestThunk, instance, token) {
  if (instance) {
    startLoader(instance.id);
  }

  authorization.set(BEARER, token);
  headers.append('Content-type', 'application/json');
  return requestThunk().then(function (data) {
    if (instance) {
      stopLoader(instance.id);
      errorHandler(instance.id, data);
    }

    return data;
  }).catch(function () {
    if (instance) {
      stopLoader(instance.id);
      onSdkAvailable(function ($sdk) {
        $sdk.dispatch({
          type: 'SET_INSTANCE_NETWORK_ERROR',
          instance: instance.id
        });
      });
    }
  });
};

export var sdkRequest = function sdkRequest(requestThunk, instance, token) {
  return requestService(requestThunk, instance, token);
};