function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

var isFunction = function isFunction(arg) {
  return typeof arg === 'function';
};

var VALID_PRIMITIVES = [String, Number, Boolean, Array, Object];

var isValidPrimitive = function isValidPrimitive(primitive) {
  return VALID_PRIMITIVES.includes(primitive);
};

var isWrappedAssertion = function isWrappedAssertion(assertion) {
  return !isValidPrimitive(assertion) && assertion !== Function && isFunction(assertion);
};

var isValidAssertion = function isValidAssertion(assertion) {
  return isValidPrimitive(assertion) || isWrappedAssertion(assertion);
};

export var NULLABLE = Symbol('NULLABLE');
export var nullable = function nullable(assertion) {
  return function () {
    return {
      type: NULLABLE,
      assertion: assertion
    };
  };
};
export var ARRAY_OF = Symbol('ARRAY_OF');
export var arrayOf = function arrayOf(primitive) {
  return function () {
    if (!isValidPrimitive(primitive)) {
      throw new Error("Invalid parameter: \"".concat(primitive, "\".") + 'It should be one of String, ' + 'Number, Boolean, Array or Object primitives.');
    }

    return {
      type: ARRAY_OF,
      primitive: primitive
    };
  };
};

var isNullable = function isNullable(arg) {
  return isWrappedAssertion(arg) && arg().type === NULLABLE;
};

var checkMatch = function checkMatch(value, assertion) {
  var type = isWrappedAssertion(assertion) ? assertion().type : undefined;
  var primitive = isWrappedAssertion(assertion) ? assertion().primitive : assertion;

  switch (type) {
    case ARRAY_OF:
      return Array.isArray(value) && value.every(function (item) {
        return checkMatch(item, primitive);
      });

    default:
      return value.constructor === primitive;
  }
};

var checkMatchWith = function checkMatchWith(obj) {
  return function (result, _ref) {
    var _ref2 = _slicedToArray(_ref, 2),
        key = _ref2[0],
        assertion = _ref2[1];

    if (!isValidAssertion(assertion)) {
      throw new Error("Invalid assertion: \"".concat(assertion, "\"."));
    }

    var isNullableAssertion = isNullable(assertion);
    var parsedAssertion = assertion;

    if (isNullable(assertion)) {
      parsedAssertion = assertion().assertion;
    }

    var partial = isNullableAssertion ? obj[key] == null || checkMatch(obj[key], parsedAssertion) : obj[key] != null && checkMatch(obj[key], parsedAssertion);
    return [].concat(_toConsumableArray(result), [partial]);
  };
};

var makeEntryNullable = function makeEntryNullable(_ref3) {
  var _ref4 = _slicedToArray(_ref3, 2),
      key = _ref4[0],
      value = _ref4[1];

  var nullableValue = isNullable(value) ? value : nullable(value);
  return [key, nullableValue];
};

var matchesInterfaceWithMap = function matchesInterfaceWithMap(mapFunc) {
  return function (interfc) {
    return function (obj) {
      var entriesMatch = Object.entries(interfc).map(mapFunc).reduce(checkMatchWith(obj), []).every(function (item) {
        return item === true;
      });
      var hasSameKeys = Object.keys(obj).every(function (objKey) {
        return interfc[objKey] != null;
      });
      return entriesMatch && hasSameKeys;
    };
  };
};

export var matchesInterface = matchesInterfaceWithMap(function (entry) {
  return entry;
});
export var matchesInterfacePartially = matchesInterfaceWithMap(makeEntryNullable);