import { replaceNonDigits } from 'sling-helpers';

export const partnersModel = (data) => {
  const modeledData = data.map(item => ({
    name: item.name,
    email: item.email,
    rg: item.rg,
    cpf: item.cpf,
  }));
  return modeledData;
};

export const contactsModel = data => data;

export const putContactsModel = (data) => {
  const modeledData = {
    name: data.name,
    typeId: Number(data.typeId),
    email: data.email,
    phone: replaceNonDigits(data.phone),
    mobilePhone: replaceNonDigits(data.mobilePhone),
  };

  return modeledData;
};

export const addressesModel = (data) => {
  const modeledData = data.map(item => ({
    description: item.typeName,
    uf: item.stateName,
    city: item.cityName,
    street: item.streetName,
    number: item.streetNumber,
    complement: item.complement,
    neighborhood: item.neighborhood,
  }));

  return modeledData;
};

export const infoModel = (data, stonecode) => {
  const modeledData = [{
    stonecode,
    cnpj: data.document,
    companyName: data.companyName,
    fantasyName: data.fantasyName,
    mccDescription: data.mccDescription,
  }];

  return modeledData;
};

export const bankInfoModel = (responses) => {
  const [availableBanks, banks] = responses;

  const modeledData = {
    banks: banks.data,
    availableBanks: availableBanks.data,
  };

  modeledData.banks = modeledData.banks.reduce((result, current) => {
    const repeatedAccountNumbers = result.map(item => item.accountNumber);
    const repeatedAgencyNumbers = result.map(item => item.agencyNumber);
    const repeatedBankIds = result.map(item => item.bankId);

    if (repeatedAccountNumbers.includes(current.accountNumber) &&
      repeatedAgencyNumbers.includes(current.agencyNumber) &&
      repeatedBankIds.includes(current.bankId)) {
      return result;
    }

    return [...result, current];
  }, []);

  return modeledData;
};

export const putMerchantsBankInfo = (data) => {
  const modeledPayload = {
    accountNumber: data.accountNumber,
    accountNumberVerificationCode: data.accountNumberVerificationCode,
    agencyNumber: data.agencyNumber,
    agencyNumberVerificationCode: data.agencyNumberVerificationCode,
    bankId: Number(data.bankId),
    statusId: Number(data.statusId),
    typeId: Number(data.typeId),
    centralizedPayment: data.centralizedPayment,
  };

  return modeledPayload;
};

export const feesModel = ([
  transactionProfilesResp,
  cardBrandsResp,
  merchantFeesResp,
]) => {
  const byTransactionProfileId = id => item => item.transactionProfileId !== id;
  const byCardBrandIdOf = item =>
    cardBrand =>
      cardBrand.id === item.cardBrandId;
  const byTransactionProfileIdOf = item =>
    transactionProfile =>
      transactionProfile.id === item.transactionProfileId;
  const boletoId = 10;
  const transactionProfiles = transactionProfilesResp.data;
  const cardBrands = cardBrandsResp.data;

  const merchantFees = merchantFeesResp.data
    .filter(byTransactionProfileId(boletoId))
    .map(merchantFee => ({
      ...merchantFee,
      cardBrandName: cardBrands.find(byCardBrandIdOf(merchantFee)).name,
      transactionProfileName: transactionProfiles
        .find(byTransactionProfileIdOf(merchantFee)).name,
    }));

  return merchantFees;
};
