export const defaultStatusChecker = response => response.ok;

export const defaultResponseParser = response => (response.status !== 204
  ? response.json()
  : response);

export const defaultBodyParser = body => JSON.stringify(body);

export const defaultErrorHandler = error => error;

export const makeRequest =
  (url, requestParams = {}, requestConfig = {}, fetchMaker = fetch) => {
    const {
      statusChecker = defaultStatusChecker,
      responseParser = defaultResponseParser,
      bodyParser = defaultBodyParser,
      errorHandler = defaultErrorHandler,
    } = requestConfig;

    const { body } = requestParams;
    let requestParamsWithParsedBody = requestParams;

    if (body != null) {
      requestParamsWithParsedBody = {
        ...requestParams,
        body: bodyParser(body),
      };
    }

    return fetchMaker(url, requestParamsWithParsedBody)
      .then((response) => {
        if (statusChecker(response) === true) {
          return responseParser(response);
        }
        throw errorHandler(response);
      })
      .catch((error) => {
        throw errorHandler(error);
      });
  };
