import { sdkRequest } from './index.js';
import * as sdk from '../../v0/src/index.js';
import { PORTAL_API_URL } from '../../v0/src/constants/constants.js';

import {
  putContactsModel,
  putMerchantsBankInfo,
} from '../../models/merchantModels.js';

export const getContacts =
  ({ stonecode: affiliationCode, apitoken }, instance) =>
    sdkRequest(() => sdk.merchants.contacts.get({ affiliationCode }),
      instance, apitoken);

export const putContacts =
  ({ stonecode: affiliationCode, apitoken, payload }, instance) => sdkRequest(
    () => sdk.merchants.contacts.put({ affiliationCode, id: payload.id },
      putContactsModel(payload)), instance, apitoken);

export const getPartners =
  ({ stonecode: affiliationCode, apitoken }, instance) => {
    sdk.headers.append('StoneCode', affiliationCode);

    // @todo: remove PORTAL_API_URL when merchant.partners uses the Gateway API
    return sdkRequest(() => sdk.merchants.partners.get(PORTAL_API_URL,
      { affiliationCode }), instance, apitoken);
  };

export const getAddresses =
  ({ stonecode: affiliationCode, apitoken }, instance) =>
    sdkRequest(() => sdk.merchants.addresses.get({ affiliationCode }),
      instance, apitoken);

export const getInfo =
  ({ stonecode: affiliationCode, apitoken }, instance) =>
    sdkRequest(() => sdk.merchants.merchant.get({ affiliationCode }),
      instance, apitoken);

export const patchInfo =
  ({ stonecode: affiliationCode, apitoken, payload }, instance) => sdkRequest(
    () => sdk.merchants.merchant.patch({ affiliationCode },
      payload), instance, apitoken);

export const getBankInfo =
  ({ stonecode: affiliationCode, apitoken }, instance) =>
    sdkRequest(() => sdk.merchants.bankAccounts.get({ affiliationCode }),
      instance, apitoken);

export const getAvailableBanks =
  ({ stonecode: affiliationCode, apitoken }, instance) =>
    sdkRequest(() => sdk.merchants.availableBanks.get({ affiliationCode }),
      instance, apitoken);

export const putBankAccounts =
  ({ stonecode: affiliationCode, apitoken, payload }, instance) => sdkRequest(
    () => sdk.merchants.bankAccounts.put({ affiliationCode, id: payload.id },
      putMerchantsBankInfo(payload)), instance, apitoken);

export const getFees =
  ({ stonecode: affiliationCode, apitoken }, instance) =>
    sdkRequest(() => sdk.merchants.fees.get({ affiliationCode }),
      instance, apitoken);
