import sinon from 'sinon';
import { service as contactsService } from './contacts.js';
import { contactsInterface } from './contactsInterface.js';

const urlParams = {
  affiliationCode: '321321321',
  contactKey: 1,
};

describe('Affiliation Contacts Module', () => {
  it('Should have methods after initialization.', () => {
    const service = contactsService();
    expect(service.get).to.be.a('function');
  });

  it('Should call get request.', () => {
    const spy = sinon.spy();

    const getRequest = () => spy;

    const service = contactsService(getRequest);

    service.get(urlParams);

    expect(spy).to.have.been.calledOnce;
  });

  it('Should call get request and return a valid URL.', () => {
    const spy = sinon.spy();

    const getRequest = (makeUrl) => {
      spy(makeUrl(urlParams));
      return () => {};
    };

    const service = contactsService(getRequest);

    service.get(urlParams);

    expect(spy).to.have.been
      .calledWith('v0/affiliation/merchants/321321321/contacts');
  });

  it('Should call get request with valid params.', () => {
    const spy = sinon.spy();

    const getRequest = () => spy;

    const service = contactsService(getRequest);

    service.get(urlParams);

    expect(spy).to.have.been.calledWith(urlParams);
  });

  it('Should call put request.', () => {
    const spy = sinon.spy();

    const putRequest = () => spy;

    const service = contactsService(undefined, putRequest);

    service.put(urlParams, contactsInterface);

    expect(spy).to.have.been.calledOnce;
  });

  it('Should call put request and return a valid URL.', () => {
    const spy = sinon.spy();

    const putRequest = (makeUrl) => {
      spy(makeUrl(urlParams));
      return () => {};
    };

    const service = contactsService(undefined, putRequest);

    service.put(urlParams, contactsInterface);

    expect(spy).to.have.been
      .calledWith('v0/affiliation/merchants/321321321/contacts/1');
  });

  it('Should call put request with valid params.', () => {
    const spy = sinon.spy();

    const putRequest = () => spy;

    const service = contactsService(undefined, putRequest);

    service.put(urlParams);

    expect(spy).to.have.been.calledWith(urlParams);
  });
});
