import sinon from 'sinon';
import { service as partnersService } from './partners.js';
import { putPartnerInterface, postPartnerInterface } from './partnersInterface.js';

const urlParams = {
  affiliationCode: '321321321',
  partnerKey: 1,
};

describe('Affiliation Partners Module', () => {
  it('Should have methods after initialization.', () => {
    const service = partnersService();
    expect(service.get).to.be.a('function');
  });

  it('Should call get request.', () => {
    const spy = sinon.spy();

    const getRequest = () => spy;

    const service = partnersService(getRequest);

    service.get(urlParams);

    expect(spy).to.have.been.calledOnce;
  });

  it('Should call get request and return a valid URL.', () => {
    const spy = sinon.spy();

    const getRequest = (makeUrl) => {
      spy(makeUrl(urlParams));
      return () => {};
    };

    const service = partnersService(getRequest);

    service.get(urlParams);

    expect(spy).to.have.been
      .calledOnceWith('v0/affiliation/merchants/321321321/partners');
  });

  it('Should call put request.', () => {
    const spy = sinon.spy();

    const putRequest = () => spy;

    const service = partnersService(undefined, putRequest);

    service.put(urlParams, putPartnerInterface);

    expect(spy).to.have.been.calledOnce;
  });

  it('Should call put request and return a valid URL.', () => {
    const spy = sinon.spy();

    const putRequest = (makeUrl) => {
      spy(makeUrl(urlParams));
      return () => {};
    };

    const service = partnersService(undefined, putRequest);

    service.put(urlParams, putPartnerInterface);

    expect(spy).to.have.been
      .calledWith('v0/affiliation/merchants/321321321/partners/1');
  });

  it('Should call put request with valid params.', () => {
    const spy = sinon.spy();

    const putRequest = () => spy;

    const service = partnersService(undefined, putRequest);

    service.put(urlParams);

    expect(spy).to.have.been.calledWith(urlParams);
  });

  it('Should call post request.', () => {
    const spy = sinon.spy();

    const postRequest = () => spy;

    const service = partnersService(undefined, undefined, postRequest);

    service.post(urlParams, postPartnerInterface);

    expect(spy).to.have.been.calledOnce;
  });

  it('Should call post request and return a valid URL.', () => {
    const spy = sinon.spy();

    const postRequest = (makeUrl) => {
      spy(makeUrl(urlParams));
      return () => {};
    };

    const service = partnersService(undefined, undefined, postRequest);

    service.post(urlParams, postPartnerInterface);

    expect(spy).to.have.been
      .calledWith('v0/affiliation/merchants/321321321/partners');
  });

  it('Should call post request with valid params.', () => {
    const spy = sinon.spy();

    const postRequest = () => spy;

    const service = partnersService(undefined, undefined, postRequest);

    service.post(urlParams);

    expect(spy).to.have.been.calledWith(urlParams);
  });
});
