import sinon from 'sinon';
import { service as financialInstitutions } from './financialInstitutions';

let urlParams;

describe('Partial Lock Credit Lock Contracts Module', () => {
  it('Should have methods after initialization.', () => {
    const service = financialInstitutions();
    expect(service.get).to.be.a('function');
  });

  describe('When all parameters are passed', () => {
    beforeEach(() => {
      urlParams = {
        document: '321321321',
        ispb: '12345678',
        compe: '123',
      };
    });

    it('Should call get request.', () => {
      const spy = sinon.spy();

      const getRequest = () => spy;

      const service = financialInstitutions(getRequest);

      service.get(urlParams);

      expect(spy).to.have.been.calledOnce;
    });

    it('Should call get request and return a valid URL.', () => {
      const spy = sinon.spy();

      const getRequest = (makeUrl) => {
        spy(makeUrl(urlParams));
        return () => {};
      };

      const service = financialInstitutions(getRequest);

      service.get(urlParams);

      expect(spy).to.have.been
        .calledWith('v0/merchants/financial-institutions?' +
          'document=321321321&ispb=12345678&compe=123');
    });

    it('Should call get request with valid params.', () => {
      const spy = sinon.spy();

      const getRequest = () => spy;

      const service = financialInstitutions(getRequest);

      service.get(urlParams);

      expect(spy).to.have.been.calledWith(urlParams);
    });
  });

  describe('When no parameters are sent', () => {
    beforeEach(() => {
      urlParams = {};
    });

    it('Should call get request.', () => {
      const spy = sinon.spy();

      const getRequest = () => spy;

      const service = financialInstitutions(getRequest);

      service.get(urlParams);

      expect(spy).to.have.been.calledOnce;
    });

    it('Should call get request and return a valid URL.', () => {
      const spy = sinon.spy();

      const getRequest = (makeUrl) => {
        spy(makeUrl(urlParams));
        return () => {};
      };

      const service = financialInstitutions(getRequest);

      service.get(urlParams);

      expect(spy).to.have.been
        .calledWith('v0/merchants/financial-institutions');
    });

    it('Should call get request with valid params.', () => {
      const spy = sinon.spy();

      const getRequest = () => spy;

      const service = financialInstitutions(getRequest);

      service.get(urlParams);

      expect(spy).to.have.been.calledWith(urlParams);
    });
  });

  describe('When only document is passed', () => {
    beforeEach(() => {
      urlParams = {
        document: '321321321',
      };
    });

    it('Should call get request.', () => {
      const spy = sinon.spy();

      const getRequest = () => spy;

      const service = financialInstitutions(getRequest);

      service.get(urlParams);

      expect(spy).to.have.been.calledOnce;
    });

    it('Should call get request and return a valid URL.', () => {
      const spy = sinon.spy();

      const getRequest = (makeUrl) => {
        spy(makeUrl(urlParams));
        return () => {};
      };

      const service = financialInstitutions(getRequest);

      service.get(urlParams);

      expect(spy).to.have.been
        .calledWith('v0/merchants/financial-institutions?document=321321321');
    });

    it('Should call get request with valid params.', () => {
      const spy = sinon.spy();

      const getRequest = () => spy;

      const service = financialInstitutions(getRequest);

      service.get(urlParams);

      expect(spy).to.have.been.calledWith(urlParams);
    });
  });

  describe('When only document and ispb are passed', () => {
    beforeEach(() => {
      urlParams = {
        document: '321321321',
        ispb: '12345678',
      };
    });

    it('Should call get request.', () => {
      const spy = sinon.spy();

      const getRequest = () => spy;

      const service = financialInstitutions(getRequest);

      service.get(urlParams);

      expect(spy).to.have.been.calledOnce;
    });

    it('Should call get request and return a valid URL.', () => {
      const spy = sinon.spy();

      const getRequest = (makeUrl) => {
        spy(makeUrl(urlParams));
        return () => {};
      };

      const service = financialInstitutions(getRequest);

      service.get(urlParams);

      expect(spy).to.have.been
        .calledWith('v0/merchants/financial-institutions' +
          '?document=321321321&ispb=12345678');
    });

    it('Should call get request with valid params.', () => {
      const spy = sinon.spy();

      const getRequest = () => spy;

      const service = financialInstitutions(getRequest);

      service.get(urlParams);

      expect(spy).to.have.been.calledWith(urlParams);
    });
  });

  describe('When only document and compe are passed', () => {
    beforeEach(() => {
      urlParams = {
        document: '321321321',
        compe: '123',
      };
    });

    it('Should call get request.', () => {
      const spy = sinon.spy();

      const getRequest = () => spy;

      const service = financialInstitutions(getRequest);

      service.get(urlParams);

      expect(spy).to.have.been.calledOnce;
    });

    it('Should call get request and return a valid URL.', () => {
      const spy = sinon.spy();

      const getRequest = (makeUrl) => {
        spy(makeUrl(urlParams));
        return () => {};
      };

      const service = financialInstitutions(getRequest);

      service.get(urlParams);

      expect(spy).to.have.been
        .calledWith('v0/merchants/financial-institutions' +
          '?document=321321321&compe=123');
    });

    it('Should call get request with valid params.', () => {
      const spy = sinon.spy();

      const getRequest = () => spy;

      const service = financialInstitutions(getRequest);

      service.get(urlParams);

      expect(spy).to.have.been.calledWith(urlParams);
    });
  });
});
