import { validateMiddleware } from './validateMiddleware.js';

describe('validateMiddleware', () => {
  it('Should not validate something that is not an object', () => {
    expect(() => validateMiddleware()).to.throw();
    expect(() => validateMiddleware(null)).to.throw();
    expect(() => validateMiddleware('string')).to.throw();
    expect(() => validateMiddleware(123)).to.throw();
    expect(() => validateMiddleware([])).to.throw();
  });

  it('Should not validate and object with properties that are not ' +
    'beforeRequest or afterRequest', () => {
    expect(() => validateMiddleware({
      before() {},
      after() {},
    })).to.throw();
  });

  it('Should not validate and object with beforeRequest ' +
    'and afterRequest properties that are not functions or promises', () => {
    expect(() => validateMiddleware({
      beforeRequest: 'before',
      afterRequest: 'after',
    })).to.throw();
  });

  it('Should validate an empty object', () => {
    expect(() => validateMiddleware({})).not.to.throw();
  });

  it('Should validate an object with a beforeRequest property ' +
    'that is a function', () => {
    expect(() => validateMiddleware({
      beforeRequest() {},
    })).not.to.throw();
  });

  it('Should validate an object with a afterRequest property ' +
    'that is a promise', () => {
    expect(() => validateMiddleware({
      afterRequest: Promise.resolve(),
    })).not.to.throw();
  });

  it('Should validate an object with a beforeRequest and ' +
    'afterRequest property that are functions or promises', () => {
    expect(() => validateMiddleware({
      beforeRequest() {},
      afterRequest: Promise.resolve(),
    })).not.to.throw();
  });
});
